@@ -10,11 +10,14 @@ import json
 import string
 
 from django.utils import timezone
+from django.utils.translation import get_language
 from django.conf import settings
 from django.contrib.gis.gdal.error import OGRException
 from django.contrib.gis.geos import GEOSException, fromstr
+from django.template.base import TemplateDoesNotExist
 from django.http import HttpResponse
 from django.core.urlresolvers import resolve
+from djappypod.odt import get_template
 
 import bs4
 import requests
@@ -131,7 +134,7 @@ def is_file_newer(path, date_update, delete_empty=True):
             os.remove(path)
         return False
 
-    modified = datetime.fromtimestamp(os.path.getmtime(path))
+    modified = datetime.utcfromtimestamp(os.path.getmtime(path))
     modified = modified.replace(tzinfo=timezone.utc)
     return modified > date_update
 
@@ -232,7 +235,7 @@ def capture_image(url, stream, **kwargs):
     download_to_stream(url, stream)
 
 
-def capture_map_image(url, destination, size=None, aspect=1.0):
+def capture_map_image(url, destination, size=None, aspect=1.0, waitfor='.leaflet-tile-loaded'):
     """Prepare aspect of the detail page
 
     It relies on JS code in MapEntity.Context
@@ -248,12 +251,12 @@ def capture_map_image(url, destination, size=None, aspect=1.0):
     printcontext['print'] = True
     serialized = json.dumps(printcontext)
     # Run head-less capture (takes time)
-    url += '?context=' + urllib2.quote(serialized)
+    url += '?lang={}&context={}'.format(get_language(), urllib2.quote(serialized))
 
     with open(destination, 'wb') as fd:
         capture_image(url, fd,
                       selector='.map-panel',
-                      waitfor='.leaflet-tile-loaded')
+                      waitfor=waitfor)
 
 
 def extract_attributes_html(url, request):
@@ -313,3 +316,24 @@ def alphabet_enumeration(length):
         enum += alphabet[i % 26]
         enums.append(enum)
     return enums
+
+
+def suffix_for(template_name_suffix, template_type, extension):
+    return "%s%s.%s" % (template_name_suffix, template_type, extension)
+
+
+def name_for(app, modelname, suffix):
+    return "%s/%s%s" % (app, modelname, suffix)
+
+
+def smart_get_template(model, suffix):
+    for appname, modelname in [(model._meta.app_label, model._meta.object_name.lower()),
+                               ("mapentity", "override"),
+                               ("mapentity", "mapentity")]:
+        try:
+            template_name = name_for(appname, modelname, suffix)
+            get_template(template_name)  # Will raise if not exist
+            return template_name
+        except TemplateDoesNotExist:
+            pass
+    return None
