@@ -1,19 +1,23 @@
 import os
+import math
 
 from django.db import models
 from django.db.utils import OperationalError
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import FieldError, ObjectDoesNotExist
+from django.core.urlresolvers import NoReverseMatch
 from django.contrib import auth
 from django.contrib.admin.models import LogEntry as BaseLogEntry
 from django.contrib.admin.models import ADDITION, CHANGE, DELETION
 from django.utils.formats import localize
+from django.utils.timezone import utc
 from django.utils.translation import ugettext_lazy as _
 from paperclip.models import Attachment
+from rest_framework import permissions as rest_permissions
 
-from mapentity.templatetags.timesince import humanize_timesince
-from . import app_settings
+from mapentity.templatetags.mapentity_tags import humanize_timesince
+from .settings import app_settings, API_SRID
 from .helpers import smart_urljoin, is_file_newer, capture_map_image, extract_attributes_html
 
 
@@ -50,13 +54,22 @@ ENTITY_PERMISSIONS = (
 )
 
 
+class MapEntityRestPermissions(rest_permissions.DjangoModelPermissions):
+    perms_map = {
+        'GET': ['%(app_label)s.read_%(model_name)s'],
+        'OPTIONS': ['%(app_label)s.read_%(model_name)s'],
+        'HEAD': ['%(app_label)s.read_%(model_name)s'],
+        'POST': ['%(app_label)s.add_%(model_name)s'],
+        'PUT': ['%(app_label)s.change_%(model_name)s'],
+        'PATCH': ['%(app_label)s.change_%(model_name)s'],
+        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
+    }
+
+
 class MapEntityMixin(object):
 
-    @classmethod
-    def add_property(cls, name, func):
-        if hasattr(cls, name):
-            return  # ignore
-        setattr(cls, name, property(func))
+    _entity = None
+    capture_map_image_waitfor = '.leaflet-tile-loaded'
 
     @classmethod
     def get_create_label(cls):
@@ -90,19 +103,24 @@ class MapEntityMixin(object):
     def get_permission_codename(cls, entity_kind):
         perm = cls.get_entity_kind_permission(entity_kind)
         opts = cls._meta
-        return '%s.%s' % (opts.app_label.lower(),
-                          auth.get_permission_codename(perm, opts))
+        appname = opts.app_label.lower()
+        if opts.proxy:
+            proxied = opts.proxy_for_model._meta
+            appname = proxied.app_label.lower()
+        return '%s.%s' % (appname, auth.get_permission_codename(perm, opts))
 
     @classmethod
     def latest_updated(cls):
         try:
-            return cls.objects.latest("date_update").date_update
+            fname = app_settings['DATE_UPDATE_FIELD_NAME']
+            return cls.objects.latest(fname).get_date_update()
         except (cls.DoesNotExist, FieldError):
             return None
 
     def get_date_update(self):
         try:
-            return self.date_update
+            fname = app_settings['DATE_UPDATE_FIELD_NAME']
+            return getattr(self, fname).replace(tzinfo=utc)
         except AttributeError:
             return None
 
@@ -118,42 +136,30 @@ class MapEntityMixin(object):
             os.unlink(image_path)
         super(MapEntityMixin, self).delete(*args, **kwargs)
 
-    @classmethod
-    def get_url_name(cls, kind):
-        if kind not in ENTITY_KINDS:
-            return None
-        return '%s:%s_%s' % (cls._meta.app_label, cls._meta.module_name, kind)
-
-    @classmethod
-    def get_url_name_for_registration(cls, kind):
-        if kind not in ENTITY_KINDS:
-            return None
-        return '%s_%s' % (cls._meta.module_name, kind)
-
     @classmethod
     @models.permalink
     def get_layer_url(cls):
-        return (cls.get_url_name(ENTITY_LAYER), )
+        return (cls._entity.url_name(ENTITY_LAYER), )
 
     @classmethod
     @models.permalink
     def get_list_url(cls):
-        return (cls.get_url_name(ENTITY_LIST), )
+        return (cls._entity.url_name(ENTITY_LIST), )
 
     @classmethod
     @models.permalink
     def get_jsonlist_url(cls):
-        return (cls.get_url_name(ENTITY_JSON_LIST), )
+        return (cls._entity.url_name(ENTITY_JSON_LIST), )
 
     @classmethod
     @models.permalink
     def get_format_list_url(cls):
-        return (cls.get_url_name(ENTITY_FORMAT_LIST), )
+        return (cls._entity.url_name(ENTITY_FORMAT_LIST), )
 
     @classmethod
     @models.permalink
     def get_add_url(cls):
-        return (cls.get_url_name(ENTITY_CREATE), )
+        return (cls._entity.url_name(ENTITY_CREATE), )
 
     def get_absolute_url(self):
         return self.get_detail_url()
@@ -161,46 +167,11 @@ class MapEntityMixin(object):
     @classmethod
     @models.permalink
     def get_generic_detail_url(cls):
-        return (cls.get_url_name(ENTITY_DETAIL), [str(0)])
+        return (cls._entity.url_name(ENTITY_DETAIL), [str(0)])
 
     @models.permalink
     def get_detail_url(self):
-        return (self.get_url_name(ENTITY_DETAIL), [str(self.pk)])
-
-    @property
-    def attachments(self):
-        return Attachment.objects.attachments_for_object(self)
-
-    def get_geom_aspect_ratio(self, maximum=None):
-        """ Returns a ratio with/height, bounded to a maximum aspect.
-        """
-        geom = self.get_geom()
-        if geom:
-            xmin, ymin, xmax, ymax = geom.extent
-            try:
-                aspect = (xmax - xmin) / (ymax - ymin)
-                if maximum is None:
-                    maximum = app_settings['MAP_CAPTURE_MAX_RATIO']
-                if maximum > 0:
-                    aspect = max(min(maximum, aspect), 1.0 / maximum)
-                return aspect
-            except ZeroDivisionError:
-                pass
-        return 1.0
-
-    def prepare_map_image(self, rooturl):
-        path = self.get_map_image_path()
-        # Do nothing if image is up-to-date
-        if is_file_newer(path, self.get_date_update()):
-            return
-        url = smart_urljoin(rooturl, self.get_detail_url())
-        capture_map_image(url, path, aspect=self.get_geom_aspect_ratio())
-
-    def get_map_image_path(self):
-        basefolder = os.path.join(settings.MEDIA_ROOT, 'maps')
-        if not os.path.exists(basefolder):
-            os.makedirs(basefolder)
-        return os.path.join(basefolder, '%s-%s.png' % (self._meta.module_name, self.pk))
+        return (self._entity.url_name(ENTITY_DETAIL), [str(self.pk)])
 
     @property
     def map_image_url(self):
@@ -208,23 +179,60 @@ class MapEntityMixin(object):
 
     @models.permalink
     def get_map_image_url(self):
-        return (self.get_url_name(ENTITY_MAPIMAGE), [str(self.pk)])
+        return (self._entity.url_name(ENTITY_MAPIMAGE), [str(self.pk)])
 
     @models.permalink
     def get_document_url(self):
-        return (self.get_url_name(ENTITY_DOCUMENT), [str(self.pk)])
+        return (self._entity.url_name(ENTITY_DOCUMENT), [str(self.pk)])
 
     @models.permalink
     def get_update_url(self):
-        return (self.get_url_name(ENTITY_UPDATE), [str(self.pk)])
+        return (self._entity.url_name(ENTITY_UPDATE), [str(self.pk)])
 
     @models.permalink
     def get_delete_url(self):
-        return (self.get_url_name(ENTITY_DELETE), [str(self.pk)])
+        return (self._entity.url_name(ENTITY_DELETE), [str(self.pk)])
 
-    def get_attributes_html(self, rooturl):
+    @property
+    def attachments(self):
+        return Attachment.objects.attachments_for_object(self)
+
+    def get_map_image_extent(self, srid=API_SRID):
+        fieldname = app_settings['GEOM_FIELD_NAME']
+        obj = getattr(self, fieldname)
+        obj.transform(srid)
+        return obj.extent
+
+    def prepare_map_image(self, rooturl):
+        if self.get_geom() is None:
+            return True
+        path = self.get_map_image_path()
+        # Do nothing if image is up-to-date
+        if is_file_newer(path, self.get_date_update()):
+            return False
         url = smart_urljoin(rooturl, self.get_detail_url())
-        return extract_attributes_html(url)
+        extent = self.get_map_image_extent(3857)
+        length = max(extent[2] - extent[0], extent[3] - extent[1])
+        if length:
+            hint_size = app_settings['MAP_CAPTURE_SIZE']
+            length_per_tile = 256 * length / hint_size
+            RADIUS = 6378137
+            CIRCUM = 2 * math.pi * RADIUS
+            zoom = round(math.log(CIRCUM / length_per_tile, 2))
+            size = math.ceil(length * 1.1 * 256 * 2 ** zoom / CIRCUM)
+        else:
+            size = app_settings['MAP_CAPTURE_SIZE']
+        capture_map_image(url, path, size=size, waitfor=self.capture_map_image_waitfor)
+        return True
+
+    def get_map_image_path(self):
+        basefolder = os.path.join(settings.MEDIA_ROOT, 'maps')
+        if not os.path.exists(basefolder):
+            os.makedirs(basefolder)
+        return os.path.join(basefolder, '%s-%s.png' % (self._meta.module_name, self.pk))
+
+    def get_attributes_html(self, request):
+        return extract_attributes_html(self.get_detail_url(), request)
 
     @classmethod
     def get_content_type_id(cls):
@@ -235,22 +243,26 @@ class MapEntityMixin(object):
 
     @property
     def creator(self):
-        log_entry = LogEntry.objects.get(
+        log_entry = LogEntry.objects.filter(
             content_type_id=self.get_content_type_id(),
             object_id=self.pk,
-            action_flag=ADDITION)
-        return log_entry.user
+            action_flag=ADDITION).order_by('pk').last()
+        return log_entry and log_entry.user
 
     @property
     def authors(self):
         return auth.get_user_model().objects.filter(
             logentry__content_type_id=self.get_content_type_id(),
-            logentry__object_id=self.pk)
+            logentry__object_id=self.pk).distinct()
 
     @property
     def last_author(self):
         return self.authors.order_by('logentry__pk').last()
 
+    def is_public(self):
+        "Override this method to allow unauthenticated access to attachments"
+        return False
+
 
 class LogEntry(MapEntityMixin, BaseLogEntry):
     geom = None
@@ -277,8 +289,10 @@ class LogEntry(MapEntityMixin, BaseLogEntry):
         model_str = unicode(self.content_type)
         try:
             obj = self.get_edited_object()
-        except ObjectDoesNotExist:
+            assert obj._entity, 'Unregistered model %s' % model_str
+            obj_url = obj.get_detail_url()
+        except (ObjectDoesNotExist, NoReverseMatch, AssertionError):
             return u'%s %s' % (model_str, self.object_repr)
         else:
             return u'<a data-pk="%s" href="%s" >%s %s</a>' % (
-                obj.pk, obj.get_detail_url(), model_str, self.object_repr)
+                obj.pk, obj_url, model_str, self.object_repr)
