@@ -9,9 +9,11 @@ import json
 import mimetypes
 
 from django.conf import settings
-from django.contrib.auth.decorators import login_required, permission_required
+from django.contrib.auth.decorators import login_required
+from django.contrib.gis.db.models import GeometryField
 from django.http import (HttpResponse, HttpResponseBadRequest,
-                         HttpResponseServerError)
+                         HttpResponseServerError, Http404)
+from django.core.exceptions import PermissionDenied
 from django.core.urlresolvers import reverse
 from django.views.defaults import page_not_found, permission_denied
 from django.views.generic.base import TemplateView
@@ -19,11 +21,15 @@ from django.views import static
 from django.views.decorators.http import require_http_methods
 from django.views.decorators.csrf import csrf_exempt
 from django.template import RequestContext, Context, loader
+from django.shortcuts import get_object_or_404
+from django.db.models.loading import get_model
 
+from ..decorators import view_permission_required
 from ..settings import app_settings, _MAP_STYLES
 from ..helpers import capture_image
-from .. import urlizor
-from .mixins import JSONResponseMixin
+from .mixins import JSONResponseMixin, FilterListMixin, ModelViewMixin
+
+from mapentity import models as mapentity_models
 
 
 logger = logging.getLogger(__name__)
@@ -60,15 +66,24 @@ def handler500(request, template_name='mapentity/500.html'):
     return HttpResponseServerError(response)
 
 
-@permission_required('paperclip.read_attachment',
-                     raise_exception=True)
-def serve_secure_media(request, path):
+def serve_attachment(request, path, app_label, model_name, pk):
     """
-    Serve media/ for authenticated users only, since it can contain sensitive
-    information (uploaded documents, map screenshots, ...)
+    Serve media/ for authorized users only, since it can contain sensitive
+    information (uploaded documents)
     """
-    if path.startswith('/'):
-        path = path[1:]
+    model = get_model(app_label, model_name)
+    if not model:
+        raise Http404
+    if not issubclass(model, mapentity_models.MapEntityMixin):
+        raise Http404
+    obj = get_object_or_404(model, pk=pk)
+    if not obj.is_public():
+        if not request.user.is_authenticated():
+            raise PermissionDenied
+        if not request.user.has_perm('paperclip.read_attachment'):
+            raise PermissionDenied
+        if not request.user.has_perm('%s.read_%s' % (app_label, model_name)):
+            raise PermissionDenied
 
     content_type, encoding = mimetypes.guess_type(path)
 
@@ -76,7 +91,7 @@ def serve_secure_media(request, path):
         response = static.serve(request, path, settings.MEDIA_ROOT)
     else:
         response = HttpResponse()
-        response['X-Accel-Redirect'] = os.path.join(settings.MEDIA_URL_SECURE, path)
+        response[app_settings['SENDFILE_HTTP_HEADER']] = os.path.join(settings.MEDIA_URL_SECURE, path)
     response["Content-Type"] = content_type or 'application/octet-stream'
     if encoding:
         response["Content-Encoding"] = encoding
@@ -106,13 +121,18 @@ class JSSettings(JSONResponseMixin, TemplateView):
         dictsettings['urls'] = {}
         dictsettings['urls']['root'] = root_url
 
-        class ModelName:
+        from django.db import models
+        from mapentity.registry import MapEntityOptions
+
+        class ModelName(mapentity_models.MapEntityMixin, models.Model):
             pass
 
+        options = MapEntityOptions(ModelName)
+
         dictsettings['urls']['static'] = settings.STATIC_URL
-        dictsettings['urls']['layer'] = root_url + urlizor.url_layer(ModelName)[1:-1]
+        dictsettings['urls']['layer'] = root_url + options._url_path(mapentity_models.ENTITY_LAYER)[1:-1]
         dictsettings['urls']['detail'] = root_url + 'modelname/0/'
-        dictsettings['urls']['format_list'] = root_url + urlizor.url_format_list(ModelName)[1:-1]
+        dictsettings['urls']['format_list'] = root_url + options._url_path(mapentity_models.ENTITY_FORMAT_LIST)[1:-1]
         dictsettings['urls']['screenshot'] = reverse("mapentity:map_screenshot")
 
         # Useful for JS calendars
@@ -123,6 +143,26 @@ class JSSettings(JSONResponseMixin, TemplateView):
         return dictsettings
 
 
+class BaseListView(FilterListMixin, ModelViewMixin):
+
+    columns = None
+
+    def __init__(self, *args, **kwargs):
+        super(BaseListView, self).__init__(*args, **kwargs)
+
+        if self.columns is None:
+            # All model fields except geometries
+            self.columns = [field.name for field in self.get_model()._meta.fields
+                            if not isinstance(field, GeometryField)]
+            # Id column should be the first one
+            self.columns.remove('id')
+            self.columns.insert(0, 'id')
+
+    @view_permission_required()
+    def dispatch(self, *args, **kwargs):
+        return super(BaseListView, self).dispatch(*args, **kwargs)
+
+
 @csrf_exempt
 @login_required
 def map_screenshot(request):
